<?php

// Generate our internal data structure and store it into a PHP code file
generateDataFile();

// Include generated file and do some tests
#include 'publicsuffixdata.inc.php';

#$t0 = microtime(true);

#echo getPublicSuffix('a.b.gov.uk') . '<br>';
#echo getPublicSuffix('a.b.t-online.de') . '<br>';

#echo '<p>' . round((microtime(true) - $t0) * 1000, 2) . ' ms</p>';

#doTest();


// ======================================  LIBRARY FUNCTIONS  =====================================


// Gets the shortest regularly registered domain name out of a given full host name.
//
// hostName = (string) Full host name to process, must be in IDN Punycode form (i.e. not Unicode UTF-8).
//
// Returns (string) Domain name that can be used as provider name, or null for invalid host names.
//
function getPublicSuffix($hostName)
{
	global $publicSuffixData;
	
	// Clean up given host name
	$hostName = strtolower($hostName);
	$hostName = trim($hostName, '.');
	$hostName = explode('.', $hostName);
	$tld = $hostName[count($hostName) - 1];
	
	// Compare both types of rules, exceptions first, then normal (positive) rules
	for ($type = 0; $type <= 1; $type++)
	{
		// Compare every rule
		foreach ($publicSuffixData[$type] as $rule)
		{
			if ($tld !== $rule[count($rule) - 1]) continue;   // Top-level domain does not match
			if (count($rule) > count($hostName)) continue;   // Rule is longer than host name
			// Compare every segment of the rule
			for ($i = 0; $i < count($rule); $i++)
			{
				$hnPart = $hostName[count($hostName) - 1 - $i];
				$rPart = $rule[count($rule) - 1 - $i];
				if ($rPart !== '*' &&
					$rPart !== $hnPart) continue 2;   // Some non-wildcard segment of the rule did not match
			}
			// All rule segments matched, finish up
			$isException = $type == 0;
			if (!$isException && count($rule) == count($hostName)) return null;   // Host name is exactly the rule
			// Combine as many parts as the rule's length plus one more as the resulting domain name
			return join('.', array_slice($hostName, -count($rule) - 1 + ($isException ? 1 : 0)));
		}
	}
	return null;
}


// ====================================  CONVERSION FUNCTIONS  ====================================


function generateDataFile()
{
	// Download the current file from Mozilla's code repository or use local copy
	if (file_exists('effective_tld_names.dat'))
		$lines = file('effective_tld_names.dat');
	else
		$lines = file('http://mxr.mozilla.org/mozilla-central/source/netwerk/dns/effective_tld_names.dat?raw=1');

	$exceptionRules = array();
	$normalRules = array();

	foreach ($lines as $rule)
	{
		// Ignore empty or comment lines
		$rule = trim($rule);
		if (!strlen($rule)) continue;
		if (substr($rule, 0, 2) == '//') continue;

		// Detect exception rules
		$exception = false;
		if ($rule{0} === '!')
		{
			$exception = true;
			$rule = substr($rule, 1);
		}

		// Convert everything from UTF-8 to IDN Punycode (requires PHP 5.3 and intl extension)
		$rule = idn_to_ascii($rule);

		// Split to levels
		$rule = explode('.', $rule);
		$levels = count($rule);

		if ($exception)
		{
			$exceptionRules[$levels][] = $rule;
		}
		else
		{
			$normalRules[$levels][] = $rule;
		}
	}

	$publicSuffixData = array();
	// $publicSuffixData[0] contains all exceptions
	// $publicSuffixData[1] contains all normal rules

	for ($i = 10; $i >= 1; $i--)
	{
		if (is_array($exceptionRules[$i]))
		{
			foreach ($exceptionRules[$i] as $rule)
			{
				$publicSuffixData[0][] = $rule;
			}
		}
	}
	for ($i = 10; $i >= 1; $i--)
	{
		if (is_array($normalRules[$i]))
		{
			foreach ($normalRules[$i] as $rule)
			{
				$publicSuffixData[1][] = $rule;
			}
		}
	}
	
	$out = "<?php\n" .
		"// This Source Code Form is subject to the terms of the Mozilla Public\n" .
		"// License, v. 2.0. If a copy of the MPL was not distributed with this\n" .
		"// file, You can obtain one at http://mozilla.org/MPL/2.0/.\n\n" .
		"// Source: http://publicsuffix.org/\n\n" .
		"// Generated by tm web analytics, (c) Yves Goergen.\n\n" .
		"\$publicSuffixData=" . generateArrayCode($publicSuffixData) . ";\n" .
		"?>";
	file_put_contents('publicsuffixdata.inc.php', $out);
}

function generateArrayCode(&$arr)
{
	$out = 'array(';
	$count = 0;
	foreach ($arr as $a)
	{
		if ($count++)
		{
			$out .= ',';
		}
		if (is_array($a))
		{
			$out .= generateArrayCode($a);
		}
		else if (is_string($a))
		{
			$out .= '\'' . addslashes($a) . '\'';
		}
		else
		{
			$out .= $a;
		}
	}
	$out .= ')';
	return $out;
}


// =========================================  TEST CASES  =========================================


// Test cases. Print errors or remain silent.
//
function doTest()
{
	// Source: http://mxr.mozilla.org/mozilla-central/source/netwerk/test/unit/data/test_psl.txt?raw=1
	// via: http://publicsuffix.org/list/
	// Licence: Public Domain
	
	// Non-sense and invalid test cases removed.
	
	// Mixed case.
	checkPublicSuffix('COM', null);
	checkPublicSuffix('example.COM', 'example.com');
	checkPublicSuffix('WwW.example.COM', 'example.com');
	// TLD with only 1 rule.
	checkPublicSuffix('biz', null);
	checkPublicSuffix('domain.biz', 'domain.biz');
	checkPublicSuffix('b.domain.biz', 'domain.biz');
	checkPublicSuffix('a.b.domain.biz', 'domain.biz');
	// TLD with some 2-level rules.
	checkPublicSuffix('com', null);
	checkPublicSuffix('example.com', 'example.com');
	checkPublicSuffix('b.example.com', 'example.com');
	checkPublicSuffix('a.b.example.com', 'example.com');
	checkPublicSuffix('uk.com', null);
	checkPublicSuffix('example.uk.com', 'example.uk.com');
	checkPublicSuffix('b.example.uk.com', 'example.uk.com');
	checkPublicSuffix('a.b.example.uk.com', 'example.uk.com');
	checkPublicSuffix('test.ac', 'test.ac');
	// TLD with only 1 (wildcard) rule.
	checkPublicSuffix('cy', null);
	checkPublicSuffix('c.cy', null);
	checkPublicSuffix('b.c.cy', 'b.c.cy');
	checkPublicSuffix('a.b.c.cy', 'b.c.cy');
	// More complex TLD.
	checkPublicSuffix('jp', null);
	checkPublicSuffix('test.jp', 'test.jp');
	checkPublicSuffix('www.test.jp', 'test.jp');
	checkPublicSuffix('ac.jp', null);
	checkPublicSuffix('test.ac.jp', 'test.ac.jp');
	checkPublicSuffix('www.test.ac.jp', 'test.ac.jp');
	checkPublicSuffix('kyoto.jp', null);
	checkPublicSuffix('test.kyoto.jp', 'test.kyoto.jp');
	checkPublicSuffix('ide.kyoto.jp', null);
	checkPublicSuffix('b.ide.kyoto.jp', 'b.ide.kyoto.jp');
	checkPublicSuffix('a.b.ide.kyoto.jp', 'b.ide.kyoto.jp');
	checkPublicSuffix('c.kobe.jp', null);
	checkPublicSuffix('b.c.kobe.jp', 'b.c.kobe.jp');
	checkPublicSuffix('a.b.c.kobe.jp', 'b.c.kobe.jp');
	checkPublicSuffix('city.kobe.jp', 'city.kobe.jp');
	checkPublicSuffix('www.city.kobe.jp', 'city.kobe.jp');
	// TLD with a wildcard rule and exceptions.
	checkPublicSuffix('om', null);
	checkPublicSuffix('test.om', null);
	checkPublicSuffix('b.test.om', 'b.test.om');
	checkPublicSuffix('a.b.test.om', 'b.test.om');
	checkPublicSuffix('songfest.om', 'songfest.om');
	checkPublicSuffix('www.songfest.om', 'songfest.om');
	// US K12.
	checkPublicSuffix('us', null);
	checkPublicSuffix('test.us', 'test.us');
	checkPublicSuffix('www.test.us', 'test.us');
	checkPublicSuffix('ak.us', null);
	checkPublicSuffix('test.ak.us', 'test.ak.us');
	checkPublicSuffix('www.test.ak.us', 'test.ak.us');
	checkPublicSuffix('k12.ak.us', null);
	checkPublicSuffix('test.k12.ak.us', 'test.k12.ak.us');
	checkPublicSuffix('www.test.k12.ak.us', 'test.k12.ak.us');
}

// Used for test cases in doTest().
//
function checkPublicSuffix($hostName, $publicSuffix)
{
	if (($result = getPublicSuffix($hostName)) !== $publicSuffix)
	{
		echo 'ERROR: ' . $hostName . ' -&gt; ' . $result . ' (expected: ' . $publicSuffix . ')<br>';
	}
}

?>
